@@ -15,7 +15,7 @@
  * "CRAM-MD5".
  */
 int
-auth_cram_md5(session * ssn, const char *user, const char *pass)
+auth_cram_md5(session *ssn, const char *user, const char *pass)
 {
 	int t;
 	size_t n;
@@ -28,39 +28,40 @@ auth_cram_md5(session * ssn, const char *user, const char *pass)
 	if ((t = imap_authenticate(ssn, "CRAM-MD5")) == -1)
 		return -1;
 
-	if (response_authenticate(ssn, t, &chal) == RESPONSE_CONTINUE) {
-		n = strlen(chal) * 3 / 4 + 1;
-		resp = (char *)xmalloc(n);
+	if (response_authenticate(ssn, t, &chal) ==
+	    STATUS_RESPONSE_CONTINUE) {
+		n = strlen((char *)(chal)) * 3 / 4 + 1;
+		resp = (unsigned char *)xmalloc(n * sizeof(char));
 		memset(resp, 0, n);
 
-		EVP_DecodeBlock(resp, chal, strlen(chal));
+		EVP_DecodeBlock(resp, chal, strlen((char *)(chal)));
 
 		HMAC_Init(&hmac, (const unsigned char *)pass, strlen(pass),
 		    EVP_md5());
-		HMAC_Update(&hmac, resp, strlen(resp));
+		HMAC_Update(&hmac, resp, strlen((char *)(resp)));
 		HMAC_Final(&hmac, md, &mdlen);
 
 		xfree(chal);
 		xfree(resp);
 
 		for (i = 0; i < mdlen; i++)
-			snprintf(mdhex + i * 2, mdlen * 2 - i * 2 + 1,
+			snprintf((char *)(mdhex) + i * 2, mdlen * 2 - i * 2 + 1,
 			    "%02x", md[i]);
 		mdhex[mdlen * 2] = '\0';
 
-		n = strlen(user) + 1 + strlen(mdhex) + 1;
-		buf = (unsigned char *)xmalloc(n);
+		n = strlen(user) + 1 + strlen((char *)(mdhex)) + 1;
+		buf = (unsigned char *)xmalloc(n * sizeof(unsigned char));
 		memset(buf, 0, n);
 
-		snprintf(buf, n, "%s %s", user, mdhex);
+		snprintf((char *)(buf), n, "%s %s", user, mdhex);
 
-		n = (strlen(buf) + 3) * 4 / 3 + 1;
-		out = (unsigned char *)xmalloc(n);
+		n = (strlen((char *)(buf)) + 3) * 4 / 3 + 1;
+		out = (unsigned char *)xmalloc(n * sizeof(unsigned char));
 		memset(out, 0, n);
 
-		EVP_EncodeBlock(out, buf, strlen(buf));
+		EVP_EncodeBlock(out, buf, strlen((char *)(buf)));
 
-		imap_continuation(ssn, out);
+		imap_continuation(ssn, (char *)(out));
 
 		xfree(buf);
 		xfree(out);
