@@ -8,10 +8,10 @@
  * Initialize buffer.
  */
 void
-buffer_init(buffer * buf, size_t n)
+buffer_init(buffer *buf, size_t n)
 {
 
-	buf->data = (char *)xmalloc(n + 1);
+	buf->data = (char *)xmalloc((n + 1) * sizeof(char));
 	*buf->data = '\0';
 	buf->size = n;
 }
@@ -21,8 +21,9 @@ buffer_init(buffer * buf, size_t n)
  * Free allocated memory of buffer.
  */
 void
-buffer_free(buffer * buf)
+buffer_free(buffer *buf)
 {
+
 	if (!buf->data)
 		return;
 
@@ -35,7 +36,7 @@ buffer_free(buffer * buf)
  * Reset buffer.
  */
 void
-buffer_reset(buffer * buf)
+buffer_reset(buffer *buf)
 {
 
 	*buf->data = '\0';
@@ -47,7 +48,7 @@ buffer_reset(buffer * buf)
  * needed.
  */
 void
-buffer_check(buffer * buf, size_t n)
+buffer_check(buffer *buf, size_t n)
 {
 
 	while (n > buf->size) {
