@@ -18,9 +18,9 @@
 extern environment env;
 
 
-int check_cert(X509 * pcert, unsigned char *pmd, unsigned int *pmdlen);
-void print_cert(X509 * cert, unsigned char *md, unsigned int *mdlen);
-int write_cert(X509 * cert);
+int check_cert(X509 *pcert, unsigned char *pmd, unsigned int *pmdlen);
+void print_cert(X509 *cert, unsigned char *md, unsigned int *mdlen);
+int write_cert(X509 *cert);
 int mismatch_cert(void);
 
 
@@ -29,7 +29,7 @@ int mismatch_cert(void);
  * accordingly.
  */
 int
-get_cert(session * ssn)
+get_cert(session *ssn)
 {
 	X509 *cert;
 	unsigned char md[EVP_MAX_MD_SIZE];
@@ -57,10 +57,12 @@ get_cert(session * ssn)
 	}
 
 	X509_free(cert);
+
 	return 0;
 
 fail:
 	X509_free(cert);
+
 	return -1;
 }
 
@@ -69,7 +71,7 @@ fail:
  * Check if the SSL/TLS certificate exists in the certificates file.
  */
 int
-check_cert(X509 * pcert, unsigned char *pmd, unsigned int *pmdlen)
+check_cert(X509 *pcert, unsigned char *pmd, unsigned int *pmdlen)
 {
 	int n, r;
 	FILE *fd;
@@ -91,8 +93,10 @@ check_cert(X509 * pcert, unsigned char *pmd, unsigned int *pmdlen)
 	certf = (char *)xmalloc((n + 1) * sizeof(char));
 	snprintf(certf, n + 1, "%s/%s", env.home, PATHNAME_CERTS);
 
-	if (!exists_file(certf))
+	if (!exists_file(certf)) {
+		xfree(certf);
 		return 0;
+	}
 
 	fd = fopen(certf, "r");
 
@@ -129,7 +133,7 @@ check_cert(X509 * pcert, unsigned char *pmd, unsigned int *pmdlen)
  * Print information about the SSL/TLS certificate.
  */
 void
-print_cert(X509 * cert, unsigned char *md, unsigned int *mdlen)
+print_cert(X509 *cert, unsigned char *md, unsigned int *mdlen)
 {
 	unsigned int i;
 	char *c;
@@ -152,7 +156,7 @@ print_cert(X509 * cert, unsigned char *md, unsigned int *mdlen)
  * Write the SSL/TLS certificate after asking the user to accept/reject it.
  */
 int
-write_cert(X509 * cert)
+write_cert(X509 *cert)
 {
 	int n;
 	FILE *fd;
