@@ -17,7 +17,7 @@ static int tag = 0x1000;	/* Every IMAP command is prefixed with a
 				 * unique [:alnum:] string. */
 
 
-int send_command(session * ssn, char *cmd, char *alt);
+int send_command(session *ssn, char *cmd, char *alt);
 void prepare_command(const char *fmt,...);
 
 
@@ -25,10 +25,8 @@ void prepare_command(const char *fmt,...);
  * Send to server data; a command.
  */
 int
-send_command(session * ssn, char *cmd, char *alt)
+send_command(session *ssn, char *cmd, char *alt)
 {
-	ssize_t w;
-	char *b;
 
 	if (ssn->socket == -1)
 		return -1;
@@ -38,9 +36,8 @@ send_command(session * ssn, char *cmd, char *alt)
 
 	verbose("C (%d): %s", ssn->socket, (alt ? alt : cmd));
 
-	for (w = 0, b = cmd; w != (ssize_t) strlen(b); b += w, w = 0)
-		if ((w = socket_write(ssn, b)) == -1)
-			return -1;
+	if (socket_write(ssn, cmd, strlen(cmd)) == -1)
+		return -1;
 
 	if (tag == 0xFFFF)	/* Tag always between 0x1000 and 0xFFFF. */
 		tag = 0x0FFF;
@@ -75,19 +72,16 @@ prepare_command(const char *fmt,...)
  * Send a response to a command continuation request.
  */
 int
-imap_continuation(session * ssn, const char *cont)
+imap_continuation(session *ssn, const char *cont)
 {
-	ssize_t w;
-	const char *b;
 
 	if (ssn->socket == -1)
 		return -1;
 
-	for (w = 0, b = cont; w != (ssize_t) strlen(b); b += w, w = 0)
-		if ((w = socket_write(ssn, b)) == -1)
-			return -1;
 
-	socket_write(ssn, "\r\n");
+	if (socket_write(ssn, cont, strlen(cont)) == -1 ||
+	    socket_write(ssn, "\r\n", strlen("\r\n")) == -1)
+		return -1;
 
 	debug("sending continuation data (%d):\n\n%s\r\n\n", ssn->socket, cont);
 
@@ -95,24 +89,11 @@ imap_continuation(session * ssn, const char *cont)
 }
 
 
-/*
- * IMAP NOOP: does nothing always succeeds.
- */
-int
-imap_noop(session * ssn)
-{
-
-	prepare_command("%04X NOOP\r\n", tag);
-
-	return send_command(ssn, obuf.data, NULL);
-}
-
-
 /*
  * IMAP CAPABILITY: requests listing of capabilities that the server supports.
  */
 int
-imap_capability(session * ssn)
+imap_capability(session *ssn)
 {
 
 	prepare_command("%04X CAPABILITY\r\n", tag);
@@ -122,14 +103,13 @@ imap_capability(session * ssn)
 
 
 /*
- * IMAP NAMESPACE: discovers the prefix and delimeter of namespaces used by the
- * server for mailboxes (RFC 2342).
+ * IMAP NOOP: does nothing always succeeds.
  */
 int
-imap_namespace(session * ssn)
+imap_noop(session *ssn)
 {
 
-	prepare_command("%04X NAMESPACE\r\n", tag);
+	prepare_command("%04X NOOP\r\n", tag);
 
 	return send_command(ssn, obuf.data, NULL);
 }
@@ -139,7 +119,7 @@ imap_namespace(session * ssn)
  * IMAP LOGOUT: informs server that client is done.
  */
 int
-imap_logout(session * ssn)
+imap_logout(session *ssn)
 {
 
 	prepare_command("%04X LOGOUT\r\n", tag);
@@ -152,7 +132,7 @@ imap_logout(session * ssn)
  * IMAP STARTTLS: begin TLS negotiation.
  */
 int
-imap_starttls(session * ssn)
+imap_starttls(session *ssn)
 {
 
 	prepare_command("%04X STARTTLS\r\n", tag);
@@ -167,7 +147,7 @@ imap_starttls(session * ssn)
  * authentication protocol exchange.
  */
 int
-imap_authenticate(session * ssn, const char *auth)
+imap_authenticate(session *ssn, const char *auth)
 {
 
 	prepare_command("%04X AUTHENTICATE %s\r\n", tag, auth);
@@ -181,7 +161,7 @@ imap_authenticate(session * ssn, const char *auth)
  * IMAP LOGIN: identifies client to server.
  */
 int
-imap_login(session * ssn, const char *user, const char *pass)
+imap_login(session *ssn, const char *user, const char *pass)
 {
 	int n, r;
 	char c;
@@ -192,7 +172,7 @@ imap_login(session * ssn, const char *user, const char *pass)
 
 	/* Alternate command with password shrouded for safe printing. */
 	n = snprintf(&c, 1, "%04X LOGIN \"%s\" *\r\n", tag, user);
-	s = (char *)xmalloc(n + 1);
+	s = (char *)xmalloc((n + 1) * sizeof(char));
 	snprintf(s, n + 1, "%04X LOGIN \"%s\" *\r\n", tag, user);
 
 	r = send_command(ssn, obuf.data, s);
@@ -203,38 +183,11 @@ imap_login(session * ssn, const char *user, const char *pass)
 }
 
 
-/*
- * IMAP SUBSCRIBE: adds the specified mailbox name to the server's set of
- * "active" or "subscribed" mailboxes.
- */
-int
-imap_subscribe(session * ssn, const char *mbox)
-{
-
-	prepare_command("%04X SUBSCRIBE \"%s\"\r\n", tag, mbox);
-
-	return send_command(ssn, obuf.data, NULL);
-}
-
-
-/*
- * IMAP EXAMINE: access a mailbox in READ-ONLY mode.
- */
-int
-imap_examine(session * ssn, const char *mbox)
-{
-
-	prepare_command("%04X EXAMINE \"%s\"\r\n", tag, mbox);
-
-	return send_command(ssn, obuf.data, NULL);
-}
-
-
 /*
  * IMAP SELECT: access a mailbox in READ-WRITE mode.
  */
 int
-imap_select(session * ssn, const char *mbox)
+imap_select(session *ssn, const char *mbox)
 {
 
 	prepare_command("%04X SELECT \"%s\"\r\n", tag, mbox);
@@ -244,13 +197,13 @@ imap_select(session * ssn, const char *mbox)
 
 
 /*
- * IMAP STATUS: requests status of the indicated mailbox.
+ * IMAP EXAMINE: access a mailbox in READ-ONLY mode.
  */
 int
-imap_status(session * ssn, const char *mbox, const char *items)
+imap_examine(session *ssn, const char *mbox)
 {
 
-	prepare_command("%04X STATUS \"%s\" (%s)\r\n", tag, mbox, items);
+	prepare_command("%04X EXAMINE \"%s\"\r\n", tag, mbox);
 
 	return send_command(ssn, obuf.data, NULL);
 }
@@ -260,7 +213,7 @@ imap_status(session * ssn, const char *mbox, const char *items)
  * IMAP CREATE: create mailbox.
  */
 int
-imap_create(session * ssn, const char *mbox)
+imap_create(session *ssn, const char *mbox)
 {
 
 	prepare_command("%04X CREATE \"%s\"\r\n", tag, mbox);
@@ -269,11 +222,108 @@ imap_create(session * ssn, const char *mbox)
 }
 
 
+/*
+ * IMAP SUBSCRIBE: adds the specified mailbox name to the server's set of
+ * "active" or "subscribed" mailboxes.
+ */
+int
+imap_subscribe(session *ssn, const char *mbox)
+{
+
+	prepare_command("%04X SUBSCRIBE \"%s\"\r\n", tag, mbox);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP LIST: returns a subset of names from the complete set of all names
+ * available.
+ */
+int
+imap_list(session *ssn, const char *refer, const char *name)
+{
+
+	prepare_command("%04X LIST \"%s\" \"%s\"\r\n", tag, refer, name);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP LSUB: returns a subset of names from the set of names that the user has
+ * declared as being "active" or "subscribed".
+ */
+int
+imap_lsub(session *ssn, const char *refer, const char *name)
+{
+
+	prepare_command("%04X LSUB \"%s\" \"%s\"\r\n", tag, refer, name);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP STATUS: requests status of the indicated mailbox.
+ */
+int
+imap_status(session *ssn, const char *mbox, const char *items)
+{
+
+	prepare_command("%04X STATUS \"%s\" (%s)\r\n", tag, mbox, items);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP APPEND: append message to the end of a mailbox.
+ */
+int
+imap_append(session *ssn, const char *mbox, const char *flags,
+    const char *date, unsigned int size)
+{
+
+	prepare_command("%04X APPEND \"%s\"%s%s%s%s%s%s {%d}\r\n", tag, mbox,
+	    (flags ? " (" : ""), (flags ? flags : ""), (flags ? ")" : ""),
+	    (date ? " \"" : ""), (date ? date : ""), (date ? "\"" : ""), size);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP CLOSE: delete messages and return to authenticated state.
+ */
+int
+imap_close(session *ssn)
+{
+
+	prepare_command("%04X CLOSE\r\n", tag);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP EXPUNGE: permanently removes any messages with the \Deleted flag set.
+ */
+int
+imap_expunge(session *ssn)
+{
+
+	prepare_command("%04X EXPUNGE\r\n", tag);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
 /*
  * IMAP SEARCH: searches the mailbox for messages that match certain criteria.
  */
 int
-imap_search(session * ssn, int uid, const char *charset, const char *criteria)
+imap_search(session *ssn, int uid, const char *charset, const char *criteria)
 {
 
 	if (charset != NULL && *charset != '\0')
@@ -291,7 +341,7 @@ imap_search(session * ssn, int uid, const char *charset, const char *criteria)
  * IMAP FETCH: retrieves data associated with a message.
  */
 int
-imap_fetch(session * ssn, int uid, const char *mesg, const char *items)
+imap_fetch(session *ssn, int uid, const char *mesg, const char *items)
 {
 
 	prepare_command("%04X %sFETCH %s %s\r\n", tag, (uid ? "UID " : ""),
@@ -305,7 +355,8 @@ imap_fetch(session * ssn, int uid, const char *mesg, const char *items)
  * IMAP STORE: alters data associated with a message.
  */
 int
-imap_store(session * ssn, int uid, const char *mesg, const char *mode, const char *flags)
+imap_store(session *ssn, int uid, const char *mesg, const char *mode,
+    const char *flags)
 {
 
 	prepare_command("%04X %sSTORE %s %sFLAGS.SILENT (%s)\r\n", tag,
@@ -320,7 +371,7 @@ imap_store(session * ssn, int uid, const char *mesg, const char *mode, const cha
  * IMAP COPY: copy messages to mailbox.
  */
 int
-imap_copy(session * ssn, int uid, const char *mesg, const char *mbox)
+imap_copy(session *ssn, int uid, const char *mesg, const char *mbox)
 {
 
 	prepare_command("%04X %sCOPY %s \"%s\"\r\n", tag, (uid ? "UID " : ""),
@@ -331,43 +382,14 @@ imap_copy(session * ssn, int uid, const char *mesg, const char *mbox)
 
 
 /*
- * IMAP APPEND: append message to the end of a mailbox.
+ * IMAP NAMESPACE: discovers the prefix and delimeter of namespaces used by the
+ * server for mailboxes (RFC 2342).
  */
 int
-imap_append(session * ssn, const char *mbox, const char *flags, const char *date,
-    unsigned int size)
+imap_namespace(session *ssn)
 {
 
-	prepare_command("%04X APPEND \"%s\"%s%s%s%s%s%s {%d}\r\n", tag, mbox,
-	    (flags ? " (" : ""), (flags ? flags : ""), (flags ? ")" : ""),
-	    (date ? " \"" : ""), (date ? date : ""), (date ? "\"" : ""), size);
-
-	return send_command(ssn, obuf.data, NULL);
-}
-
-
-
-/*
- * IMAP CLOSE: delete messages and return to authenticated state.
- */
-int
-imap_close(session * ssn)
-{
-
-	prepare_command("%04X CLOSE\r\n", tag);
-
-	return send_command(ssn, obuf.data, NULL);
-}
-
-
-/*
- * IMAP EXPUNGE: permanently removes any messages with the \Deleted flag set.
- */
-int
-imap_expunge(session * ssn)
-{
-
-	prepare_command("%04X EXPUNGE\r\n", tag);
+	prepare_command("%04X NAMESPACE\r\n", tag);
 
 	return send_command(ssn, obuf.data, NULL);
 }
