@@ -9,19 +9,20 @@
 
 #include "imapfilter.h"
 #include "session.h"
-#include "queue.h"
+#include "list.h"
 #include "version.h"
 #include "buffer.h"
 #include "pathnames.h"
+#include "regexp.h"
 
 
 extern buffer ibuf, obuf, nbuf;
+extern regexp responses[];
 
 options opts;			/* Program options. */
 environment env;		/* Environment variables. */
 
-struct sessionhead sessions = XLIST_HEAD_INITIALIZER(sessions);	/* Active IMAP sessions. */
-
+list *sessions = NULL;		/* Active IMAP sessions. */
 
 void usage(void);
 void version(void);
@@ -39,13 +40,15 @@ main(int argc, char *argv[])
 
 	opts.debug = 0;
 	opts.verbose = 0;
+	opts.interactive = 0;
 	opts.log = NULL;
 	opts.config = NULL;
+	opts.oneline = NULL;
 
 	env.home = getenv("HOME");
 	env.pathmax = -1;
 
-	while ((c = getopt(argc, argv, "Vc:dl:v?")) != -1) {
+	while ((c = getopt(argc, argv, "Vc:de:il:v?")) != -1) {
 		switch (c) {
 		case 'V':
 			version();
@@ -57,6 +60,12 @@ main(int argc, char *argv[])
 			if (opts.debug < 2)
 				opts.debug++;
 			break;
+		case 'e':
+			opts.oneline = optarg;
+			break;
+		case 'i':
+			opts.interactive = 1;
+			break;
 		case 'l':
 			opts.log = optarg;
 			break;
@@ -80,8 +89,8 @@ main(int argc, char *argv[])
 
 	open_log();
 
-	buffer_init(&ibuf, RESPONSE_BUF);
-	buffer_init(&obuf, REQUEST_BUF);
+	buffer_init(&ibuf, INPUT_BUF);
+	buffer_init(&obuf, OUTPUT_BUF);
 	buffer_init(&nbuf, NAMESPACE_BUF);
 
 	if (opts.config == NULL) {
@@ -99,12 +108,20 @@ main(int argc, char *argv[])
 		snprintf(opts.config, n + 1, "%s/%s", env.home,
 		    PATHNAME_CONFIG);
 	}
+
+	regexp_compile(responses);
+
 	start_lua();
 
 	{
-		session *s, *t;
+		list *l;
+		session *s;
+
+		l = sessions;
+		while (l != NULL) {
+			s = l->data;
+			l = l->next;
 
-		XLIST_FOREACH_SAFE(s, &sessions, links, t) {
 			response_generic(s, imap_logout(s));
 			close_connection(s);
 			session_destroy(s);
@@ -113,6 +130,8 @@ main(int argc, char *argv[])
 
 	stop_lua();
 
+	regexp_free(responses);
+
 	buffer_free(&ibuf);
 	buffer_free(&obuf);
 	buffer_free(&nbuf);
@@ -131,8 +150,8 @@ void
 usage(void)
 {
 
-	fprintf(stderr, "usage: imapfilter [-Vdv] [-c configfile] "
-	    "[-l logfile]\n");
+	fprintf(stderr, "usage: imapfilter [-Vdiv] [-c configfile] "
+	    "[-e 'command'] [-l logfile]\n");
 
 	exit(0);
 }
