@@ -10,14 +10,14 @@
 
 #include "imapfilter.h"
 #include "session.h"
-#include "queue.h"
+#include "list.h"
 #include "pathnames.h"
 
 
 extern options opts;
 extern environment env;
 extern unsigned int flags;
-extern struct sessionhead sessions;
+extern list *sessions;
 
 static FILE *debugfp = NULL;	/* Pointer to debug file. */
 static FILE *logfp = NULL;	/* Pointer to log file. */
@@ -90,7 +90,8 @@ void
 fatal(unsigned int errnum, const char *fmt,...)
 {
 	va_list args;
-	session *s, *t;
+	list *l;
+	session *s;
 
 	va_start(args, fmt);
 
@@ -99,8 +100,10 @@ fatal(unsigned int errnum, const char *fmt,...)
 
 	va_end(args);
 
-	XLIST_FOREACH_SAFE(s, &sessions, links, t)
-	    close_connection(s);
+	for (l = sessions; l; l = l->next) {
+		s = l->data;
+		close_connection(s);
+	}
 
 	close_log();
 	close_debug();
