@@ -17,6 +17,7 @@ static lua_State *lua;		/* Lua interpreter state. */
 
 
 void init_options(void);
+void interactive_mode(void);
 
 
 /*
@@ -44,14 +45,26 @@ start_lua()
 
 	init_options();
 
-	if (luaL_loadfile(lua, PATHNAME_INTERFACE) || lua_pcall(lua, 0, LUA_MULTRET, 0))
+	if (luaL_loadfile(lua, PATHNAME_INTERFACE) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
 		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
 
-	if (luaL_loadfile(lua, PATHNAME_AUXILIARY) || lua_pcall(lua, 0, LUA_MULTRET, 0))
+	if (luaL_loadfile(lua, PATHNAME_AUXILIARY) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
 		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
 
-	if (luaL_loadfile(lua, opts.config) || lua_pcall(lua, 0, LUA_MULTRET, 0))
-		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+	if (opts.oneline != NULL) {
+		if (luaL_loadbuffer(lua, opts.oneline, strlen(opts.oneline),
+		    "=<command line>") || lua_pcall(lua, 0, LUA_MULTRET, 0))
+			fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+	} else {
+		if (luaL_loadfile(lua, opts.config) ||
+		    lua_pcall(lua, 0, LUA_MULTRET, 0))
+			fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+	}
+
+	if (opts.interactive)
+		interactive_mode();
 }
 
 
@@ -61,6 +74,7 @@ start_lua()
 void
 stop_lua(void)
 {
+
 	lua_close(lua);
 }
 
@@ -88,6 +102,32 @@ init_options(void)
 }
 
 
+/*
+ * Interactive mode.
+ */
+void
+interactive_mode(void)
+{
+	char buf[LINE_MAX];
+	
+	for (;;) {
+		printf("> ");
+		fflush(stdout);
+
+		if (fgets(buf, sizeof(buf), stdin) == NULL) {
+			printf("\n");
+			break;
+		}
+
+		if (luaL_loadbuffer(lua, buf, strlen(buf), "=<line>") ||
+		    lua_pcall(lua, 0, LUA_MULTRET, 0)) {
+			error("%s\n", lua_tostring(lua, -1));
+			lua_pop(lua, 1);
+		}
+	}
+}
+
+
 /*
  * Get from the configuration file the value of a boolean option variable.
  */
@@ -226,6 +266,7 @@ get_table_string(const char *key)
 int
 set_table_nil(const char *key)
 {
+
 	lua_pushstring(lua, key);
 	lua_pushnil(lua);
 	lua_settable(lua, -3);
