@@ -8,7 +8,7 @@
 /*
  * An implementation of strstr() with case-insensitivity.
  */
-char *
+const char *
 xstrcasestr(const char *haystack, const char *needle)
 {
 	const char *h, *n, *c;
@@ -35,7 +35,7 @@ xstrcasestr(const char *haystack, const char *needle)
 			n++;
 
 			if (*n == '\0')
-				return (char *)c;
+				return c;
 		}
 		c++;
 		hl--;
@@ -45,22 +45,21 @@ xstrcasestr(const char *haystack, const char *needle)
 }
 
 
-#if 0
 /*
  * Copies at most size characters from the string pointed by src to the array
  * pointed by dest, always NULL terminating (unless size == 0).  Returns
  * pointer to dest.
  */
 char *
-xstrncpy(char *dest, const char *src, size_t size)
+xstrncpy(char *dst, const char *src, size_t len)
 {
 	char *d;
 	const char *s;
 	size_t n;
 
-	d = dest;
+	d = dst;
 	s = src;
-	n = size;
+	n = len;
 
 	while (n != 0) {
 		if ((*d++ = *s++) == '\0')
@@ -68,9 +67,8 @@ xstrncpy(char *dest, const char *src, size_t size)
 		n--;
 	}
 
-	if (n == 0 && size != 0)
+	if (n == 0 && len != 0)
 		*d = '\0';
 
-	return dest;
+	return dst;
 }
-#endif
