@@ -18,7 +18,7 @@
 #define PATHNAME_CERTS			PATHNAME_HOME "/certificates"
 
 /* Debug temporary file template. */
-#define PATHNAME_DEBUG			"/tmp/imapfilter.XXXXXX"
+#define PATHNAME_DEBUG			PATHNAME_HOME "/debug.XXXXXX"
 
 
 #endif				/* PATHNAMES_H */
