@@ -9,11 +9,32 @@
 
 extern options opts;
 
-buffer nbuf;			/* Namespace buffer. */
+
+int create_mailbox(session *ssn, const char *mbox);
 
 
-int create_mailbox(session * ssn, const char *mbox);
-const char *apply_namespace(const char *mbox, char *prefix, char delim);
+/*
+ * Reset any inactivity autologout timer on the server.
+ */
+int
+request_noop(const char *server, const char *user)
+{
+	int r;
+	session *s;
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	if ((r = response_generic(s, imap_noop(s))) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
 
 
 /*
@@ -28,7 +49,7 @@ request_login(const char *server, const char *user, const char *pass,
 	session *s;
 
 	if ((s = session_find(server, user)))
-		return RESPONSE_OK;
+		return STATUS_RESPONSE_OK;
 
 	s = session_new();
 
@@ -62,7 +83,7 @@ request_login(const char *server, const char *user, const char *pass,
 	if (!ssl && s->capabilities & CAPABILITY_STARTTLS &&
 	    get_option_boolean("starttls"))
 		switch (response_generic(s, imap_starttls(s))) {
-		case RESPONSE_OK:
+		case STATUS_RESPONSE_OK:
 			if (open_secure_connection(s, "tls1") == -1)
 				goto fail;
 			if (response_capability(s, imap_capability(s)) == -1)
@@ -74,8 +95,8 @@ request_login(const char *server, const char *user, const char *pass,
 		}
 #endif
 
-	r = RESPONSE_NONE;
-	if (rg != RESPONSE_PREAUTH) {
+	r = STATUS_RESPONSE_NONE;
+	if (rg != STATUS_RESPONSE_PREAUTH) {
 #ifndef NO_CRAMMD5
 		if (s->capabilities & CAPABILITY_CRAMMD5 &&
 		    get_option_boolean("crammd5")) {
@@ -83,11 +104,11 @@ request_login(const char *server, const char *user, const char *pass,
 				goto fail;
 		}
 #endif
-		if (r != RESPONSE_OK && (r = response_generic(s, imap_login(s, user,
-		    pass))) == -1)
+		if (r != STATUS_RESPONSE_OK &&
+		    (r = response_generic(s, imap_login(s, user, pass))) == -1)
 			goto fail;
 
-		if (r == RESPONSE_NO) {
+		if (r == STATUS_RESPONSE_NO) {
 			error("username %s or password rejected at %s\n",
 			    user, server);
 			goto fail;
@@ -98,10 +119,12 @@ request_login(const char *server, const char *user, const char *pass,
 		if (response_namespace(s, imap_namespace(s)) == -1)
 			goto fail;
 	}
+
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -134,21 +157,24 @@ int
 request_status(const char *server, const char *user, const char *mbox,
     unsigned int *exists, unsigned int *recent, unsigned int *unseen)
 {
-	int r;
+	int t, r;
 	session *s;
+	const char *m;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
+	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
+
+	*exists = *recent = *unseen = -1;
+
 	if (s->protocol == PROTOCOL_IMAP4REV1) {
-		if ((r = response_status(s, imap_status(s,
-		    apply_namespace(mbox, s->ns.prefix, s->ns.delim),
-		    "MESSAGES RECENT UNSEEN"), exists, recent, unseen)) == -1)
+		t = imap_status(s, m, "MESSAGES RECENT UNSEEN");
+		if ((r = response_status(s, t, exists, recent, unseen)) == -1)
 			goto fail;
 	} else {
-		if ((r = response_examine(s, imap_examine(s,
-		    apply_namespace(mbox, s->ns.prefix, s->ns.delim)), exists,
-		    recent, unseen)) == -1)
+		t = imap_examine(s, m);
+		if ((r = response_examine(s, t, exists, recent)) == -1)
 			goto fail;
 	}
 
@@ -156,6 +182,7 @@ request_status(const char *server, const char *user, const char *mbox,
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -168,18 +195,21 @@ request_select(const char *server, const char *user, const char *mbox)
 {
 	int r;
 	session *s;
+	const char *m;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
-	if ((r = response_select(s, imap_select(s,
-	    apply_namespace(mbox, s->ns.prefix, s->ns.delim)))) == -1)
+	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
+
+	if ((r = response_select(s, imap_select(s, m))) == -1)
 		goto fail;
 
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -203,6 +233,65 @@ request_close(const char *server, const char *user)
 fail:
 	close_connection(s);
 	session_destroy(s);
+
+	return -1;
+}
+
+
+/*
+ * List available mailboxes.
+ */
+int
+request_list(const char *server, const char *user, const char *refer,
+    const char *name, char **mboxs, char **folders)
+{
+	int t, r;
+	session *s;
+	const char *n;
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	n = apply_namespace(name, s->ns.prefix, s->ns.delim);
+	
+	t = imap_list(s, refer, n);
+	if ((r = response_list(s, t, mboxs, folders)) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
+/*
+ * List subscribed mailboxes.
+ */
+int
+request_lsub(const char *server, const char *user, const char *refer,
+    const char *name, char **mboxs, char **folders)
+{
+	int t, r;
+	session *s;
+	const char *n;
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	n = apply_namespace(name, s->ns.prefix, s->ns.delim);
+
+	t = imap_lsub(s, refer, n);
+	if ((r = response_lsub(s, t, mboxs, folders)) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
 	return -1;
 }
 
@@ -214,45 +303,134 @@ int
 request_search(const char *server, const char *user, const char *criteria,
     char **mesgs)
 {
-	int r;
+	int t, r;
 	session *s;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
-	if ((r = response_search(s, imap_search(s, get_option_boolean("uid"),
-	    get_option_string("charset"), criteria), mesgs)) == -1)
+	t = imap_search(s, get_option_boolean("uid"),
+	    get_option_string("charset"), criteria);
+	if ((r = response_search(s, t, mesgs)) == -1)
 		goto fail;
 
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
 
 /*
- * Fetch the specified information or data of the messages.
+ * Fetch the FLAGS, INTERNALDATE and RFC822.SIZE of the messages.
  */
 int
-request_fetch(const char *server, const char *user, const char *mesg,
-    const char *items, char **fetch)
+request_fetchfast(const char *server, const char *user, const char *mesg,
+    char **flags, char **date, char **size)
 {
-	int r;
+	int t, r;
 	session *s;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
-	if ((r = response_fetch(s, imap_fetch(s, get_option_boolean("uid"),
-	    mesg, items), fetch)) == -1)
+	t = imap_fetch(s, get_option_boolean("uid"), mesg, "FAST");
+	if ((r = response_fetchfast(s, t, flags, date, size)) == -1)
 		goto fail;
 
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
+	return -1;
+}
+
+
+/*
+ * Fetch the header, ie. BODY[HEADER], of the messages.
+ */
+int
+request_fetchheader(const char *server, const char *user, const char *mesg,
+    char **header, unsigned long int *len)
+{
+	int t, r;
+	session *s;
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	t = imap_fetch(s, get_option_boolean("uid"), mesg, "BODY.PEEK[HEADER]");
+	if ((r = response_fetchheader(s, t, header, len)) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
+/*
+ * Fetch the body, ie. BODY[TEXT], of the messages.
+ */
+int
+request_fetchbody(const char *server, const char *user, const char *mesg,
+    char **body, unsigned long int *len)
+{
+	int t, r;
+	session *s;
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	t = imap_fetch(s, get_option_boolean("uid"), mesg, "BODY.PEEK[TEXT]");
+	if ((r = response_fetchbody(s, t, body, len)) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
+/*
+ * Fetch the specified header fields, ie. BODY[HEADER.FIELDS ()], of the
+ * messages.
+ */
+int
+request_fetchfields(const char *server, const char *user, const char *mesg,
+    const char *headerfields, char **fields, unsigned long int *len)
+{
+	int t, r, n;
+	session *s;
+	char *f;
+
+	n = strlen("BODY.PEEK[HEADER.FIELDS ()]") + strlen(headerfields) + 1;
+	f = (char *)xmalloc(n * sizeof(char));
+	snprintf(f, n, "%s%s%s", "BODY.PEEK[HEADER.FIELDS (", headerfields, ")]");
+
+	if (!(s = session_find(server, user)))
+		return -1;
+
+	t = imap_fetch(s, get_option_boolean("uid"), mesg, f);
+	if ((r = response_fetchfields(s, t, fields, len)) == -1)
+		goto fail;
+
+	xfree(f);
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
 	return -1;
 }
 
@@ -264,14 +442,14 @@ int
 request_store(const char *server, const char *user, const char *mesg,
     const char *mode, const char *flags)
 {
-	int r;
+	int t, r;
 	session *s;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
-	if ((r = response_generic(s, imap_store(s, get_option_boolean("uid"),
-	    mesg, mode, flags))) == -1)
+	t = imap_store(s, get_option_boolean("uid"), mesg, mode, flags);
+	if ((r = response_generic(s, t)) == -1)
 		goto fail;
 
 	if (xstrcasestr(flags, "\\Deleted") && get_option_boolean("expunge"))
@@ -282,6 +460,7 @@ request_store(const char *server, const char *user, const char *mesg,
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -293,17 +472,19 @@ int
 request_copy(const char *server, const char *user, const char *mesg,
     const char *mbox)
 {
-	int r;
+	int t, r;
 	session *s;
+	const char *m;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
+	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
+
 	do {
-		switch (r = response_generic(s, imap_copy(s,
-		    get_option_boolean("uid"), mesg, apply_namespace(mbox,
-		    s->ns.prefix, s->ns.delim)))) {
-		case RESPONSE_TRYCREATE:
+		t = imap_copy(s, get_option_boolean("uid"), mesg, m);
+		switch (r = response_generic(s, t)) {
+		case STATUS_RESPONSE_TRYCREATE:
 			if (create_mailbox(s, mbox) == -1)
 				goto fail;
 			break;
@@ -311,12 +492,13 @@ request_copy(const char *server, const char *user, const char *mesg,
 			goto fail;
 			break;
 		}
-	} while (r == RESPONSE_TRYCREATE);
+	} while (r == STATUS_RESPONSE_TRYCREATE);
 
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -328,28 +510,29 @@ int
 request_append(const char *server, const char *user, const char *mbox,
     const char *mesg, const char *flags, const char *date)
 {
-	int r;
-	int t;
+	int t, r;
 	session *s;
+	const char *m;
 
 	if (!(s = session_find(server, user)))
 		return -1;
 
+	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
+
 	do {
-		if ((t = imap_append(s, apply_namespace(mbox, s->ns.prefix,
-		    s->ns.delim), flags, date, strlen(mesg))) == -1)
+		if ((t = imap_append(s, m, flags, date, strlen(mesg))) == -1)
 			goto fail;
 		if ((r = response_generic(s, t)) == -1)
 			goto fail;
 
 		switch (r) {
-		case RESPONSE_CONTINUE:
+		case STATUS_RESPONSE_CONTINUE:
 			if (imap_continuation(s, mesg) == -1)
 				goto fail;
 			if ((r = response_generic(s, t)) == -1)
 				goto fail;
 			break;
-		case RESPONSE_TRYCREATE:
+		case STATUS_RESPONSE_TRYCREATE:
 			if (create_mailbox(s, mbox) == -1)
 				goto fail;
 			break;
@@ -357,12 +540,13 @@ request_append(const char *server, const char *user, const char *mbox,
 			goto fail;
 			break;
 		}
-	} while (r == RESPONSE_TRYCREATE);
+	} while (r == STATUS_RESPONSE_TRYCREATE);
 
 	return r;
 fail:
 	close_connection(s);
 	session_destroy(s);
+
 	return -1;
 }
 
@@ -371,52 +555,20 @@ fail:
  * Auxiliary function to create a mailbox.
  */
 int
-create_mailbox(session * ssn, const char *mbox)
+create_mailbox(session *ssn, const char *mbox)
 {
 	int r;
+	const char *m;
 
-	if ((r = response_generic(ssn, imap_create(ssn,
-	    apply_namespace(mbox, ssn->ns.prefix, ssn->ns.delim)))) == -1)
+	m = apply_namespace(mbox, ssn->ns.prefix, ssn->ns.delim);
+
+	if ((r = response_generic(ssn, imap_create(ssn, m))) == -1)
 		return -1;
 
 	if (get_option_boolean("subscribe"))
-		if (response_generic(ssn, imap_subscribe(ssn,
-		    apply_namespace(mbox, ssn->ns.prefix,
-		    ssn->ns.delim))) == -1)
+		if (response_generic(ssn, imap_subscribe(ssn, m)) == -1)
 			return -1;
 
 	return r;
 }
 
-
-/*
- * Convert the names of personal mailboxes using the namespace specified
- * by the mail server.
- */
-const char *
-apply_namespace(const char *mbox, char *prefix, char delim)
-{
-	int n;
-	char *c;
-
-	if ((prefix == NULL && delim == '\0') ||
-	    (prefix == NULL && delim == '/') ||
-	    !strcasecmp(mbox, "INBOX"))
-		return mbox;
-
-	buffer_reset(&nbuf);
-	n = snprintf(nbuf.data, nbuf.size + 1, "%s%s", (prefix ? prefix : ""),
-	    mbox);
-	if (n > (int)nbuf.size) {
-		buffer_check(&nbuf, n);
-		snprintf(nbuf.data, nbuf.size + 1, "%s%s",
-		    (prefix ? prefix : ""), mbox);
-	}
-	c = nbuf.data;
-	while ((c = strchr(c, '/')))
-		*(c++) = delim;
-
-	debug("mailbox: '%s'\n", nbuf.data);
-
-	return nbuf.data;
-}
