@@ -24,7 +24,7 @@
  * Connect to mail server.
  */
 int
-open_connection(session * ssn, const char *server, unsigned int port,
+open_connection(session *ssn, const char *server, unsigned int port,
     const char *sprotocol)
 {
 	struct sockaddr_in sa;
@@ -81,7 +81,7 @@ open_connection(session * ssn, const char *server, unsigned int port,
  * Initialize SSL/TLS connection.
  */
 int
-open_secure_connection(session * ssn, const char *protocol)
+open_secure_connection(session *ssn, const char *protocol)
 {
 	int e;
 	SSL_CTX *ctx;
@@ -125,23 +125,6 @@ fail:
 
 	return -1;
 }
-
-
-/*
- * Get description of cipher used in the SSL/TLS connection.
- *
-char *
-get_cipher(session * ssn)
-{
-	SSL_CIPHER *cipher;
-
-	if (!ssn->ssl)
-		return NULL;
-
-	cipher = SSL_get_current_cipher(ssn->ssl);
-
-	return SSL_CIPHER_description(cipher, NULL, 0);
-}*/
 #endif				/* NO_SSLTLS */
 
 
@@ -149,7 +132,7 @@ get_cipher(session * ssn)
  * Disconnect from mail server.
  */
 int
-close_connection(session * ssn)
+close_connection(session *ssn)
 {
 	int r;
 
@@ -175,13 +158,15 @@ close_connection(session * ssn)
  * Shutdown SSL/TLS connection.
  */
 int
-close_secure_connection(session * ssn)
+close_secure_connection(session *ssn)
 {
+
 	if (ssn->ssl) {
 		SSL_shutdown(ssn->ssl);
 		SSL_free(ssn->ssl);
 		ssn->ssl = NULL;
 	}
+
 	return 0;
 }
 #endif
@@ -191,7 +176,7 @@ close_secure_connection(session * ssn)
  * Read data from socket.
  */
 ssize_t
-socket_read(session * ssn, char *buf, size_t len)
+socket_read(session *ssn, char *buf, size_t len)
 {
 	int f, s, t;
 	ssize_t r;
@@ -221,8 +206,8 @@ socket_read(session * ssn, char *buf, size_t len)
 
 #ifndef NO_SSLTLS
 	if (ssn->ssl) {
-		while (SSL_pending(ssn->ssl) > 0 ||
-		    ((s = select(ssn->socket + 1, &fds, NULL, NULL, tvp)) > 0 &&
+		while (SSL_pending(ssn->ssl) > 0 || ((s =
+		    select(ssn->socket + 1, &fds, NULL, NULL, tvp)) > 0 &&
 		    FD_ISSET(ssn->socket, &fds))) {
 			r = (ssize_t) SSL_read(ssn->ssl, buf, len);
 
@@ -268,9 +253,11 @@ socket_read(session * ssn, char *buf, size_t len)
 		error("timeout period expired while waiting to read data\n");
 		goto fail;
 	}
+
 	return r;
 fail:
 	close_connection(ssn);
+
 	return -1;
 
 }
@@ -280,17 +267,17 @@ fail:
  * Write data to socket.
  */
 ssize_t
-socket_write(session * ssn, const char *buf)
+socket_write(session *ssn, const char *buf, size_t len)
 {
 	int f, s, t;
-	ssize_t w;
+	ssize_t w, wt;
 	fd_set fds;
 
 	struct timeval tv;
 
 	struct timeval *tvp = NULL;
 
-	w = 0;
+	w = wt = 0;
 	s = 1;
 
 	t = (int)(get_option_number("timeout"));
@@ -305,43 +292,53 @@ socket_write(session * ssn, const char *buf)
 	FD_ZERO(&fds);
 	FD_SET(ssn->socket, &fds);
 
+	while (len) {
 #ifndef NO_SSLTLS
-	if (ssn->ssl) {
-		while ((s = select(ssn->socket + 1, NULL, &fds, NULL, tvp) > 0 &&
-		    FD_ISSET(ssn->socket, &fds))) {
-			w = (ssize_t) SSL_write(ssn->ssl, buf, strlen(buf));
+		if (ssn->ssl) {
+			while ((s = select(ssn->socket + 1, NULL, &fds, NULL,
+			    tvp) > 0 && FD_ISSET(ssn->socket, &fds))) {
+				w = (ssize_t) SSL_write(ssn->ssl, buf, len);
 
-			if (w > 0)
-				break;
+				if (w > 0)
+					break;
 
-			switch (SSL_get_error(ssn->ssl, w)) {
-			case SSL_ERROR_WANT_READ:
-			case SSL_ERROR_WANT_WRITE:
-				continue;
-			case SSL_ERROR_ZERO_RETURN:
+				switch (SSL_get_error(ssn->ssl, w)) {
+				case SSL_ERROR_WANT_READ:
+				case SSL_ERROR_WANT_WRITE:
+					continue;
+				case SSL_ERROR_ZERO_RETURN:
+					goto fail;
+				case SSL_ERROR_SYSCALL:
+				case SSL_ERROR_SSL:
+					error("writing data; %s\n",
+					    ERR_error_string(ERR_get_error(),
+					    NULL));
+					goto fail;
+				default:
+					error("undefined ssl error "
+					    "while writing data\n");
+					goto fail;
+				}
+			}
+		} else
+#endif
+		{
+			if ((s = select(ssn->socket + 1, NULL, &fds, NULL,
+			    tvp)) > 0 && FD_ISSET(ssn->socket, &fds))
+				w = write(ssn->socket, buf, len);
+
+			if (w == -1) {
+				error("writing data; %s", strerror(errno));
 				goto fail;
-			case SSL_ERROR_SYSCALL:
-			case SSL_ERROR_SSL:
-				error("writing data; %s\n",
-				    ERR_error_string(ERR_get_error(), NULL));
-				goto fail;
-			default:
-				error("undefined ssl error while writing data\n");
+			} else if (w == 0) {
 				goto fail;
 			}
 		}
-	} else
-#endif
-	{
-		if ((s = select(ssn->socket + 1, NULL, &fds, NULL, tvp)) > 0 &&
-		    FD_ISSET(ssn->socket, &fds))
-			w = write(ssn->socket, buf, strlen(buf));
 
-		if (w == -1) {
-			error("writing data; %s", strerror(errno));
-			goto fail;
-		} else if (w == 0) {
-			goto fail;
+		if (w > 0) {
+			len -= w;
+			buf += w;
+			wt += w;
 		}
 	}
 
@@ -354,8 +351,10 @@ socket_write(session * ssn, const char *buf)
 		error("timeout period expired while waiting to write data\n");
 		goto fail;
 	}
-	return w;
+
+	return wt;
 fail:
 	close_connection(ssn);
+
 	return -1;
 }
